@@ -21,6 +21,7 @@
 #import "MEGANode.h"
 #import "megaapi.h"
 #import "PasswordNodeData.h"
+#import "MEGACreditCardNodeData+init.h"
 #import "MEGAStringList+init.h"
 #import "MEGATOTPData+init.h"
 
@@ -102,6 +103,12 @@ using namespace mega;
     return passwordNodeData;
 }
 
+- (nullable MEGACreditCardNodeData *)creditCardNodeData {
+    if (!self.megaNode || self.megaNode->getCreditCardData() == nil) return nil;
+    
+    return [[MEGACreditCardNodeData alloc] initWithMegaCreditCardNodeData:self.megaNode->getCreditCardData() cMemoryOwn:YES];
+}
+
 - (NSInteger)duration {
     return self.megaNode ? self.megaNode->getDuration() : -1;
 }
@@ -292,10 +299,18 @@ using namespace mega;
     return self.megaNode ? self.megaNode->isNodeKeyDecrypted() : NO;
 }
 
+- (BOOL)isPasswordManagerNode {
+    return self.megaNode ? self.megaNode->isPasswordManagerNode() : false;
+}
+
 - (BOOL)isPasswordNode {
     return self.megaNode ? self.megaNode->isPasswordNode() : NO;
 }
 
+- (BOOL)isCreditCardNode {
+    return self.megaNode ? self.megaNode->isCreditCardNode() : NO;
+}
+
 - (MEGAStringList *)tags {
     return self.megaNode ? [[MEGAStringList alloc] initWithMegaStringList:self.megaNode->getTags() cMemoryOwn:YES] : nil;
 }
