@@ -57,11 +57,13 @@ void QTMegaGlobalListener::onEvent(MegaApi *api, MegaEvent *e)
     QCoreApplication::postEvent(this, event, INT_MIN);
 }
 
+#ifdef ENABLE_SYNC
 void QTMegaGlobalListener::onGlobalSyncStateChanged(MegaApi *api)
 {
     QTMegaEvent *event = new QTMegaEvent(api, (QEvent::Type)QTMegaEvent::OnGlobalSyncStateChanged);
     QCoreApplication::postEvent(this, event, INT_MIN);
 }
+#endif
 
 void QTMegaGlobalListener::customEvent(QEvent *e)
 {
@@ -83,9 +85,11 @@ void QTMegaGlobalListener::customEvent(QEvent *e)
         case QTMegaEvent::OnEvent:
             if(listener) listener->onEvent(event->getMegaApi(), event->getEvent());
             break;
+#ifdef ENABLE_SYNC
         case QTMegaEvent::OnGlobalSyncStateChanged:
             if(listener) listener->onGlobalSyncStateChanged(event->getMegaApi());
             break;
+#endif
         default:
             break;
     }
