@@ -298,6 +298,7 @@ struct DemoAppFolder : public DemoApp
 #include <mega/autocomplete.h>
 
 void exec_apiurl(autocomplete::ACState& s);
+void exec_useragent(autocomplete::ACState& s);
 void exec_login(autocomplete::ACState& s);
 void exec_begin(autocomplete::ACState& s);
 void exec_signup(autocomplete::ACState& s);
@@ -349,6 +350,16 @@ void exec_getua(autocomplete::ACState& s);
 void exec_putua(autocomplete::ACState& s);
 void exec_delua(autocomplete::ACState& s);
 void exec_alerts(autocomplete::ACState& s);
+void exec_alerts_new(autocomplete::ACState& s);
+void exec_alerts_old(autocomplete::ACState& s);
+void exec_alerts_number(autocomplete::ACState& s);
+void exec_alerts_notify(autocomplete::ACState& s);
+void exec_alerts_seen(autocomplete::ACState& s);
+void exec_alerts_test_reminder(autocomplete::ACState& s);
+void exec_alerts_test_payment(autocomplete::ACState& s);
+void exec_alerts_test_payment_v2(autocomplete::ACState& s);
+void exec_alerts_add_reminder(autocomplete::ACState& s);
+void exec_alerts_add_payment(autocomplete::ACState& s);
 void exec_recentactions(autocomplete::ACState& s);
 void exec_recentnodes(autocomplete::ACState& s);
 void exec_killsession(autocomplete::ACState& s);
@@ -391,6 +402,8 @@ void exec_chatlj(autocomplete::ACState& s);
 void exec_setmaxdownloadspeed(autocomplete::ACState& s);
 void exec_setmaxuploadspeed(autocomplete::ACState& s);
 void exec_setmaxloglinesize(autocomplete::ACState& s);
+void exec_setlogjson(autocomplete::ACState& s);
+void exec_getlogjson(autocomplete::ACState& s);
 void exec_handles(autocomplete::ACState& s);
 void exec_httpsonly(autocomplete::ACState& s);
 void exec_mfac(autocomplete::ACState& s);
@@ -408,6 +421,7 @@ void exec_nodeTag(autocomplete::ACState& s);
 void exec_treecompare(autocomplete::ACState& s);
 void exec_querytransferquota(autocomplete::ACState& s);
 void exec_metamac(autocomplete::ACState& s);
+void exec_compare_file_and_node(autocomplete::ACState& s);
 void exec_resetverifiedphonenumber(autocomplete::ACState& s);
 void exec_banner(autocomplete::ACState& s);
 void exec_drivemonitor(autocomplete::ACState& s);
@@ -452,3 +466,6 @@ void exec_getpricing(autocomplete::ACState&);
 void exec_collectAndPrintTransferStats(autocomplete::ACState&);
 void exec_hashcash(autocomplete::ACState&);
 void exec_getmyip(autocomplete::ACState& s);
+void exec_dnsservers(autocomplete::ACState& s);
+// This functionality remove all nodes from vault except My backups folder and Password Manager node
+void exec_cleanVault(autocomplete::ACState& s);
