@@ -25,12 +25,15 @@
 #include "account.h"
 #include "http.h"
 #include "json.h"
+#include "network_connectivity_test_helpers.h"
 #include "node.h"
 #include "nodemanager.h"
+#include "setandelement.h"
 #include "textchat.h"
 #include "types.h"
 
 #include <memory>
+#include <optional>
 #include <variant>
 
 namespace mega {
@@ -51,7 +54,13 @@ protected:
     bool mRead = false;// if json has already been read
 
     bool loadIpsFromJson(std::vector<string>& ips, JSON& json);
-    bool cacheresolvedurls(const std::vector<string>& urls, std::vector<string>&& ips);
+    void cacheresolvedurls(const std::string& command,
+                           const std::vector<string>& urls,
+                           const std::vector<string>& ips);
+
+    // True if the command can be sent to the API lockless CS channel
+    // Updated internally by each command.
+    bool mLockless = false;
 
 public:
     MegaClient* client; // non-owning
@@ -62,14 +71,14 @@ public:
     // some commands can only succeed if they are in their own batch.  eg. smss, when the account is blocked pending validation
     bool batchSeparately;
 
-    // true if the command processing has been updated to use the URI v3 system, where successful state updates arrive via actionpackets.
-    bool mV3 = true;
+    // Returns true if the command can use the lockless CS channel
+    bool isLockless() const;
 
     // true if the command returns strings, arrays or objects, but a seqtag is (optionally) also required. In example: ["seqtag"/error, <JSON from before v3>]
     bool mSeqtagArray = false;
 
     // filters for JSON parsing in streaming
-    std::map<std::string, std::function<bool(JSON *)>> mFilters;
+    std::map<std::string, JSONSplitter::FilterCallback> mFilters;
 
     void cmd(const char*);
     void notself(MegaClient*);
@@ -346,10 +355,6 @@ public:
     bool procresult(Result, JSON&) override;
 
     CommandSetKeyPair(MegaClient*, const byte*, unsigned, const byte*, unsigned);
-
-private:
-    std::unique_ptr<byte[]> privkBuffer;
-    unsigned len;
 };
 
 // set visibility
@@ -458,21 +463,21 @@ public:
 
     bool procresult(Result, JSON&) override;
 };
+#endif
 
 class MEGA_API CommandSendDevCommand : public Command
 {
 public:
-    bool procresult(Result, JSON&) override;
+    bool procresult(Result result, JSON& json) override;
 
-    CommandSendDevCommand(MegaClient*,
+    CommandSendDevCommand(MegaClient* client,
                           const char* command,
                           const char* email = NULL,
-                          long long = 0,
-                          int = 0,
-                          int = 0,
-                          const char* = nullptr);
+                          long long q = 0,
+                          int bs = 0,
+                          int us = 0,
+                          const char* cp = nullptr);
 };
-#endif
 
 class MEGA_API CommandGetUserEmail : public Command
 {
@@ -509,7 +514,7 @@ protected:
     // sequence-tag, saved temporary while processing the response (it's received before nodes)
     string mSt;
 
-    std::unique_lock<mutex> mNodeTreeIsChanging;
+    std::unique_lock<recursive_mutex> mNodeTreeIsChanging;
     bool mFirstChunkProcessed = false;
 };
 
@@ -812,6 +817,10 @@ public:
 
 protected:
     void parseUserAttribute(JSON& json, std::string& value, std::string &version, bool asciiToBinary = true);
+    bool updatePrivateEncryptedUserAttribute(User* u,
+                                             const std::string& value,
+                                             const std::string& version,
+                                             mega::attr_t at);
     std::function<void(string*, string*, string*, error)> mCompletion;
 };
 
@@ -963,7 +972,7 @@ class MEGA_API CommandGetPH : public Command
 {
     handle ph;
     byte key[FILENODEKEYLENGTH];
-    int op; //  (op=0 -> download, op=1 fetch data, op=2 import welcomePDF)
+    int op; // (op=0 -> download, op=1 fetch data)
     bool havekey;
 
 public:
@@ -996,7 +1005,7 @@ class MEGA_API CommandEnumerateQuotaItems : public Command
 public:
     bool procresult(Result, JSON&) override;
 
-    CommandEnumerateQuotaItems(MegaClient*);
+    CommandEnumerateQuotaItems(const std::optional<std::string>& countryCode, MegaClient*);
 };
 
 class MEGA_API CommandSubmitPurchaseReceipt : public Command
@@ -1210,14 +1219,6 @@ public:
     CommandConfirmEmailLink(MegaClient*, const char*, const char *, const byte *, bool);
 };
 
-class MEGA_API CommandGetVersion : public Command
-{
-public:
-    bool procresult(Result, JSON&) override;
-
-    CommandGetVersion(MegaClient*, const char*);
-};
-
 class MEGA_API CommandGetLocalSSLCertificate : public Command
 {
 public:
@@ -1240,6 +1241,7 @@ public:
     bool procresult(Result, JSON&) override;
 
     CommandChatCreate(MegaClient*, bool group, bool publicchat, const userpriv_vector*, const string_map* ukm = NULL, const char* title = NULL, bool meetingRoom = false, int chatOptions = ChatOptions::kEmpty, const ScheduledMeeting* schedMeeting = nullptr);
+    ~CommandChatCreate();
 };
 
 typedef std::function<void(Error)> CommandSetChatOptionsCompletion;
@@ -1442,15 +1444,6 @@ public:
     CommandGetMegaAchievements(MegaClient*, AchievementsDetails *details, bool registered_user = true);
 };
 
-class MEGA_API CommandGetWelcomePDF : public Command
-{
-public:
-    bool procresult(Result, JSON&) override;
-
-    CommandGetWelcomePDF(MegaClient*);
-};
-
-
 class MEGA_API CommandMediaCodecs : public Command
 {
 public:
@@ -1702,20 +1695,31 @@ public:
     CommandDismissBanner(MegaClient*, int id, m_time_t ts);
 };
 
-
 //
 // Sets and Elements
 //
-
 class CommandSE : public Command // intermediary class to avoid code duplication
 {
 protected:
-    bool procjsonobject(JSON& json, handle& id, m_time_t& ts, handle* u, m_time_t* cts = nullptr,
-                        handle* s = nullptr, int64_t* o = nullptr, handle* ph = nullptr,
+    bool procjsonobject(JSON& json,
+                        handle& id,
+                        m_time_t& ts,
+                        handle* u,
+                        m_time_t* cts = nullptr,
+                        handle* s = nullptr,
+                        int64_t* o = nullptr,
+                        PublicLinkSet* publicLinkSet = nullptr,
                         uint8_t* setType = nullptr) const;
-    bool procresultid(JSON& json, const Result& r, handle& id, m_time_t& ts, handle* u,
-                      m_time_t* cts = nullptr, handle* s = nullptr, int64_t* o = nullptr,
-                      handle* ph = nullptr, uint8_t* setType = nullptr) const;
+    bool procresultid(JSON& json,
+                      const Result& r,
+                      handle& id,
+                      m_time_t& ts,
+                      handle* u,
+                      m_time_t* cts = nullptr,
+                      handle* s = nullptr,
+                      int64_t* o = nullptr,
+                      PublicLinkSet* publicLinkSet = nullptr,
+                      uint8_t* setType = nullptr) const;
     bool procerrorcode(const Result& r, Error& e) const;
     bool procExtendedError(JSON& json, int64_t& errCode, handle& eid) const;
 };
@@ -1932,9 +1936,12 @@ public:
 class VpnCluster
 {
 public:
-    VpnCluster(std::string&& host, std::vector<std::string>&& dns):
+    VpnCluster(std::string&& host,
+               std::vector<std::string>&& dns,
+               std::vector<std::string>&& addBlockingDns):
         mHost{std::move(host)},
-        mDns{std::move(dns)}
+        mDns{std::move(dns)},
+        mAdBlockingDns{std::move(addBlockingDns)}
     {}
 
     const std::string& getHost() const
@@ -1947,9 +1954,15 @@ public:
         return mDns;
     }
 
+    const std::vector<std::string>& getAdBlockingDns() const
+    {
+        return mAdBlockingDns;
+    }
+
 private:
-    std::string mHost; // "nz.vpn.mega.nz"
+    std::string mHost;
     std::vector<std::string> mDns; // {"8.8.8.8", "8.8.4.4", ...}
+    std::vector<std::string> mAdBlockingDns;
 };
 
 class VpnRegion
@@ -2100,6 +2113,23 @@ public:
 private:
     Cb mCompletion;
 };
+
+class MEGA_API CommandGetNetworkConnectivityTestServerInfo: public Command
+{
+public:
+    using Completion = std::function<void(const Error&, NetworkConnectivityTestServerInfo&&)>;
+    CommandGetNetworkConnectivityTestServerInfo(MegaClient*, Completion&&);
+    bool procresult(Result, JSON&) override;
+
+private:
+    void onParseFailure()
+    {
+        if (mCompletion)
+            mCompletion(API_EINTERNAL, {});
+    }
+
+    Completion mCompletion;
+};
 /* MegaVPN Commands END*/
 
 typedef std::function<void(const Error&, const std::map<std::string, std::string>& creditCardInfo)> CommandFetchCreditCardCompletion;
@@ -2266,6 +2296,46 @@ private:
     Cb mCompletion;
 };
 
+class MEGA_API CommandSetThrottlingParams: public Command
+{
+public:
+    struct ThrottlingParamsFromAPI
+    {
+        m_off_t updateRateInSeconds{-1};
+        m_off_t maxUploadsBeforeThrottle{-1};
+        m_off_t uploadCounterInactivityTime{-1};
+    };
+
+    using ResultVariant = std::variant<Error, ThrottlingParamsFromAPI>;
+    using Completion = std::function<void(const ResultVariant&)>;
+
+    CommandSetThrottlingParams(const MegaClient&, Completion&& completion);
+    bool procresult(Result, JSON&) override;
+
+private:
+    static std::pair<bool /* parseOk */, std::optional<ThrottlingParamsFromAPI>>
+        parseJson(JSON& json);
+
+    Completion mCompletion;
+};
+
+class MEGA_API CommandGetSubscriptionCancellationDetails: public Command
+{
+public:
+    using CompletionCallback = std::function<void(const Error& /*e*/,
+                                                  string&& /*originalTransactionId*/,
+                                                  int /*expiresDate*/,
+                                                  int /*cancelledDate*/)>;
+    CommandGetSubscriptionCancellationDetails(MegaClient*,
+                                              const char* originalTransactionId,
+                                              unsigned int gatewayId,
+                                              CompletionCallback&& completion = nullptr);
+    bool procresult(Result, JSON&) override;
+
+private:
+    CompletionCallback mCompletion;
+};
+
 } // namespace
 
 #endif
