@@ -214,6 +214,16 @@ public:
         return mExcludeSensitive != BoolFilter::disabled;
     }
 
+    void includeVersions(const bool includeVersions)
+    {
+        mIncludeVersions = includeVersions;
+    }
+
+    bool includeVersions() const
+    {
+        return mIncludeVersions;
+    }
+
     bool isValidNodeType(const nodetype_t nodeType) const;
     bool isValidCreationTime(const int64_t time) const;
     bool isValidModificationTime(const int64_t time) const;
@@ -240,6 +250,7 @@ private:
     TextPattern mTagFilter;
     bool mTagFilterContainsSeparator{false};
     bool mUseAndForTextQuery{true};
+    bool mIncludeVersions{false};
 
     static bool isDocType(const MimeType_t t);
 };
@@ -288,7 +299,9 @@ public:
     std::shared_ptr<Node> getNodeByHandle(NodeHandle handle);
 
     // read children from DB and load them in memory
-    sharedNode_list getChildren(const Node *parent, CancelToken cancelToken = CancelToken());
+    sharedNode_list getChildren(const Node* parent,
+                                CancelToken cancelToken = CancelToken(),
+                                bool includeVersions = false);
 
     sharedNode_vector getChildren(const NodeSearchFilter& filter, int order, CancelToken cancelFlag, const NodeSearchPage& page);
 
@@ -363,7 +376,9 @@ public:
     std::shared_ptr<Node> getNodeFromBlob(const string* nodeSerialized);
 
     // attempt to apply received keys to decrypt node's keys
-    void applyKeys(uint32_t appliedKeys);
+    void applyKeys();
+
+    void addNodePendingApplykey(std::shared_ptr<Node> node);
 
     // add node to the notification queue
     void notifyNode(std::shared_ptr<Node> node, sharedNode_vector* nodesToReport = nullptr);
@@ -415,6 +430,9 @@ public:
     // This method only can be used in Megacli for testing purposes
     uint64_t getNumberNodesInRam() const;
 
+    // Return number of nodes have had a successful applykey()
+    long long getNumNodesKeyApplied() const;
+
     // Add new relationship between parent and child
     void addChild(NodeHandle parent, NodeHandle child, Node *node);
     // remove relationship between parent and child
@@ -436,6 +454,9 @@ public:
     // This method is called when initial fetch nodes is finished
     // Initialize node counters and create indexes at DB
     void initCompleted();
+    // Drop indexes used for search funtionalities
+    // These indexes aren't required in some apps (S4)
+    void dropSearchDBIndexes();
 
     std::shared_ptr<Node> getNodeFromNodeManagerNode(NodeManagerNode& nodeManagerNode);
 
@@ -444,6 +465,9 @@ public:
     void increaseNumNodesInRam();
     void decreaseNumNodesInRam();
 
+    void increaseNumNodesAppliedKey();
+    void decreaseNumNodesAppliedKey();
+
     uint64_t getCacheLRUMaxSize() const;
     void setCacheLRUMaxSize(uint64_t cacheLRUMaxSize);
 
@@ -453,6 +477,8 @@ public:
     // i.e., when nodes have been fully loaded from a fetchnodes or from cache
     bool ready();
 
+    bool isFromRootNodeType(const Node& node) const;
+
 private:
     class NoKeyLogger
     {
@@ -524,6 +550,12 @@ private:
     // nodes that have changed and are pending to notify to app and dump to DB
     sharedNode_vector mNodeNotify;
 
+    // Stores nodes pending key application
+    std::list<std::shared_ptr<Node>> mNodePendingApplyKeys;
+
+    // tracks how many nodes have had a successful applykey()
+    std::atomic<long long> mAppliedKeyNodeCount{0};
+
     shared_ptr<Node> getNodeInRAM(NodeHandle handle);
     void saveNodeInRAM(std::shared_ptr<Node> node, bool isRootnode, MissingParentNodes& missingParentNodes);    // takes ownership
 
@@ -603,7 +635,8 @@ private:
 
     std::shared_ptr<Node> getNodeByHandle_internal(NodeHandle handle);
     sharedNode_list getChildren_internal(const Node* parent,
-                                         CancelToken cancelToken = CancelToken());
+                                         CancelToken cancelToken = CancelToken(),
+                                         bool includeVersions = false);
 
     sharedNode_vector getNodesByFingerprint_internal(const FileFingerprint& fingerprint);
     sharedNode_vector getNodesByOrigFingerprint_internal(const std::string& fingerprint, Node *parent);
@@ -618,7 +651,7 @@ private:
     void removeChanges_internal();
     void cleanNodes_internal();
     std::shared_ptr<Node> getNodeFromBlob_internal(const string* nodeSerialized);
-    void applyKeys_internal(uint32_t appliedKeys);
+    void applyKeys_internal();
     void notifyNode_internal(std::shared_ptr<Node> node, sharedNode_vector* nodesToReport);
     bool loadNodes_internal();
     uint64_t getNodeCount_internal();
