@@ -58,8 +58,11 @@ enum attr_t
     ATTR_LAST_READ_NOTIFICATION = 44, // private, non-encrypted - char array - versioned
     ATTR_LAST_ACTIONED_BANNER = 45, // private, non-encrypted - char array - versioned
     ATTR_ENABLE_TEST_SURVEYS = 46, // private - non-encrypted - char array - non-versioned
-    ATTR_WELCOME_PDF_COPIED = 47, // private - non-encrypted - char array
+    // ATTR_WELCOME_PDF_COPIED = 47, // (obsolete) private - non-encrypted - char array
     ATTR_SYNC_DESIRED_STATE = 48, // private - byte array - versioned
+    ATTR_S4 = 49, // private - non-encrypted - char array (1/0) <-- TBD
+    ATTR_S4_CONTAINER = 50, // private - non-encrypted - char array (B64 node's handle) <-- TBD
+    ATTR_DEV_OPT = 51, // private - encrypted - byte array
 };
 
 enum UserAttributeScope : char
