@@ -22,13 +22,13 @@ public:
     MountResult add(const MountInfo& info) override;
 
     // Check if a file exists in the cache.
-    bool cached(NormalizedPath path) const override;
+    bool cached(common::NormalizedPath path) const override;
 
     // Called by the client when its view of the cloud is current.
     void current() override;
 
     // Describe the inode representing the file at the specified path.
-    ErrorOr<InodeInfo> describe(const NormalizedPath& path) const override;
+    common::ErrorOr<InodeInfo> describe(const common::NormalizedPath& path) const override;
 
     // Disable an enabled mount.
     void disable(MountDisabledCallback callback,
@@ -50,7 +50,7 @@ public:
     bool enabled(const std::string& name) const override;
 
     // Execute a function on some task.
-    Task execute(std::function<void(const Task&)> function) override;
+    common::Task execute(std::function<void(const common::Task&)> function) override;
 
     // Update a mount's flags.
     MountResult flags(const std::string& name,
@@ -66,16 +66,16 @@ public:
     MountInfoVector get(bool onlyEnabled) const override;
 
     // Retrieve the path of the mount associated with name.
-    NormalizedPath path(const std::string& name) const override;
+    common::NormalizedPath path(const std::string& name) const override;
 
     // Remove a disabled mount from the database.
     MountResult remove(const std::string& path) override;
 
     // Check whether the specified path is "syncable."
-    bool syncable(const NormalizedPath& path) const override;
+    bool syncable(const common::NormalizedPath& path) const override;
 
     // Called by the client when nodes have been changed in the cloud.
-    void updated(NodeEventQueue& events) override;
+    void updated(common::NodeEventQueue& events) override;
 
     // Update the FUSE database to the specified version.
     MountResult upgrade(const LocalPath& path,
