@@ -18,21 +18,18 @@
  * You should have received a copy of the license along with this
  * program.
  */
+#include "mega/json.h"
+
+#include "mega/base64.h"
+#include "mega/logging.h"
+#include "mega/mega_utf8proc.h"
+#include "mega/megaclient.h"
+
 #include <cctype>
 #include <cstdint>
 
-#include "mega/json.h"
-#include "mega/base64.h"
-#include "mega/megaclient.h"
-#include "mega/logging.h"
-#include "mega/mega_utf8proc.h"
-
 namespace mega {
 
-std::atomic<bool> gLogJSONRequests{false};
-
-#define JSON_verbose if (gLogJSONRequests) LOG_verbose
-
 // store array or object in string s
 // reposition after object
 bool JSON::storeobject(string* s)
@@ -363,7 +360,8 @@ bool JSON::storebinary(string* dst)
     {
         const char* ptr;
 
-        if (!(ptr = strchr(pos + 1, '"')))
+        ptr = strchr(pos + 1, '"');
+        if (!ptr)
         {
             LOG_err << "Parse error (storebinary)";
             return false;
@@ -449,6 +447,11 @@ m_off_t JSON::getint()
     if ((*ptr < '0' || *ptr > '9') && *ptr != '-')
     {
         LOG_err << "Parse error (getint)";
+        // An event about failing to parse the json cannot be sent because no MegaClient instance is
+        // accessible from here.
+        assert(false && "JSON::getint(): Unexpected value in JSON");
+        // It should probably return something less common in this case, like
+        // std::numeric_limits<m_off_t>::min().
         return -1;
     }
 
@@ -745,7 +748,8 @@ void JSON::copystring(string* s, const char* p)
     {
         const char* pp;
 
-        if ((pp = strchr(p, '"')))
+        pp = strchr(p, '"');
+        if (pp)
         {
             s->assign(p, static_cast<size_t>(pp - p));
         }
@@ -1094,7 +1098,7 @@ void JSONSplitter::clear()
     mFailed = false;
 }
 
-m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)> > *filters, const char *data)
+m_off_t JSONSplitter::processChunk(std::map<string, FilterCallback>* filters, const char* data)
 {
     if (hasFailed() || hasFinished())
     {
@@ -1108,7 +1112,7 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
         {
             JSON jsonData("");
             auto& callback = filterit->second;
-            if (!callback(&jsonData))
+            if (CallbackResult::SUCCESS != callback(&jsonData))
             {
                 LOG_err << "Error starting the processing of a chunk";
             }
@@ -1131,7 +1135,7 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
             {
                 JSON jsonData("");
                 auto& callback = filterit->second;
-                if (!callback(&jsonData))
+                if (CallbackResult::SUCCESS != callback(&jsonData))
                 {
                     LOG_err << "Parsing error processing first streaming filter"
                             << " Data: " << data;
@@ -1143,10 +1147,10 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
         mStarting = false;
     }
 
-    JSON_verbose << "JSON starting processChunk at path " << mCurrentPath
-                 << " ExpectValue: " << mExpectValue << " LastName: " << mLastName
-                 << " Data: " << std::string(data, strlen(data) < 32 ? strlen(data) : 32)
-                 << " Start: " << std::string(mPos, strlen(mPos) < 32 ? strlen(mPos) : 32);
+    JSON_CHUNK_PROCESSING << "JSON starting processChunk at path " << mCurrentPath
+                          << " ExpectValue: " << mExpectValue << " LastName: " << mLastName
+                          << " Data: " << std::string(data, strlen(data) < 32 ? strlen(data) : 32)
+                          << " Start: " << std::string(mPos, strlen(mPos) < 32 ? strlen(mPos) : 32);
 
     while (*mPos)
     {
@@ -1163,7 +1167,7 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
             mStack.push_back(c + mLastName);
             mCurrentPath.append(mStack.back());
 
-            JSON_verbose << "JSON starting path: " << mCurrentPath;
+            JSON_CHUNK_PROCESSING << "JSON starting path: " << mCurrentPath;
             if (filters && filters->find(mCurrentPath) != filters->end())
             {
                 // a filter is configured for this path - recurse
@@ -1213,12 +1217,13 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
                 auto filterit = filters->find(filter);
                 if (filterit != filters->end() && filterit->second)
                 {
-                    JSON_verbose << "JSON object/array callback for path: " << filter << " Data: "
-                                 << std::string(mLastPos, static_cast<size_t>(mPos - mLastPos));
+                    JSON_CHUNK_PROCESSING
+                        << "JSON object/array callback for path: " << filter
+                        << " Data: " << std::string(mLastPos, static_cast<size_t>(mPos - mLastPos));
 
                     JSON jsonData(mLastPos);
                     auto& callback = filterit->second;
-                    if (!callback(&jsonData))
+                    if (CallbackResult::SUCCESS != callback(&jsonData))
                     {
                         LOG_err << "Parsing error processing streaming filter: " << filter
                                 << " Data: "
@@ -1230,10 +1235,13 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
                     // Callbacks should consume the exact amount of JSON, except the last one
                     if (mCurrentPath != "{" && jsonData.pos != mPos)
                     {
-                        // I'm not aborting the parsing here because no errors were detected during the processing
-                        // so probably the callback just ignored some data that it didn't need.
-                        // Anyway it would be good to check this when it happens to fix it.
-                        LOG_warn << (mPos - jsonData.pos) << " bytes were not processed by the following streaming filter: " << filter;
+                        // I'm not aborting the parsing here because no errors were detected during
+                        // the processing so probably the callback just ignored some data that it
+                        // didn't need. Anyway it would be good to check this when it happens to fix
+                        // it.
+                        LOG_warn << (mPos - jsonData.pos)
+                                 << " bytes were not processed by the following streaming filter: "
+                                 << filter;
                         assert(false);
                     }
 
@@ -1241,7 +1249,7 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
                 }
             }
 
-            JSON_verbose << "JSON finishing path: " << mCurrentPath;
+            JSON_CHUNK_PROCESSING << "JSON finishing path: " << mCurrentPath;
             mCurrentPath.resize(mCurrentPath.size() - mStack.back().size());
             mStack.pop_back();
             mExpectValue = 0;
@@ -1277,8 +1285,8 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
             int t = strEnd();
             if (t < 0)
             {
-                JSON_verbose << "JSON chunk finished parsing a string."
-                             << " Data: " << mPos;
+                JSON_CHUNK_PROCESSING << "JSON chunk finished parsing a string."
+                                      << " Data: " << mPos;
                 break;
             }
 
@@ -1290,26 +1298,33 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
                     auto filterit = filters->find(filter);
                     if (filterit != filters->end() && filterit->second)
                     {
-                        JSON_verbose << "JSON string value callback for: " << filter
-                                     << " Data: " << std::string(mPos, static_cast<size_t>(t));
+                        JSON_CHUNK_PROCESSING
+                            << "JSON string value callback for: " << filter
+                            << " Data: " << std::string(mPos, static_cast<size_t>(t));
 
                         JSON jsonData(mPos);
                         auto& callback = filterit->second;
-                        if (!callback(&jsonData))
+                        auto result = callback(&jsonData);
+                        if (result == CallbackResult::FAILED)
                         {
                             LOG_err << "Parsing error processing streaming filter: " << filter
                                     << " Data: " << std::string(mPos, static_cast<size_t>(t));
                             parseError(filters);
                             return 0;
                         }
-
+                        else if (result == CallbackResult::PAUSED)
+                        {
+                            auto consumedBytes = mLastPos - data;
+                            mProcessedBytes = mPos - mLastPos;
+                            return consumedBytes;
+                        }
                         mLastPos = mPos + t;
                     }
                 }
 
-                JSON_verbose << "JSON string value parsed at path " << mCurrentPath
-                             << " Data: " << mLastName << " = "
-                             << std::string(mPos + 1, static_cast<size_t>(t - 2));
+                JSON_CHUNK_PROCESSING << "JSON string value parsed at path " << mCurrentPath
+                                      << " Data: " << mLastName << " = "
+                                      << std::string(mPos + 1, static_cast<size_t>(t - 2));
 
                 mPos += t;
                 mExpectValue = 0;
@@ -1330,8 +1345,9 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
                     return 0;
                 }
 
-                JSON_verbose << "JSON property name parsed at path " << mCurrentPath
-                             << " Data: " << std::string(mPos + 1, static_cast<size_t>(t - 2));
+                JSON_CHUNK_PROCESSING
+                    << "JSON property name parsed at path " << mCurrentPath
+                    << " Data: " << std::string(mPos + 1, static_cast<size_t>(t - 2));
 
                 mLastName = std::string(mPos + 1, static_cast<size_t>(t - 2));
                 mPos += t + 1;
@@ -1350,13 +1366,14 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
             int j = numEnd();
             if (j < 0 || !mPos[j])
             {
-                JSON_verbose << "JSON chunk finished parsing a number."
-                             << " Data: " << mPos;
+                JSON_CHUNK_PROCESSING << "JSON chunk finished parsing a number."
+                                      << " Data: " << mPos;
                 break;
             }
 
-            JSON_verbose << "JSON number parsed at path " << mCurrentPath << " Data: " << mLastName
-                         << " = " << std::string(mPos, static_cast<size_t>(j));
+            JSON_CHUNK_PROCESSING << "JSON number parsed at path " << mCurrentPath
+                                  << " Data: " << mLastName << " = "
+                                  << std::string(mPos, static_cast<size_t>(j));
 
             mPos += j;
             mExpectValue = 0;
@@ -1373,11 +1390,13 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
                     if (filterit != filters->end())
                     {
                         JSON jsonData(mLastPos);
-                        JSON_verbose << "JSON error callback."
-                                     << " Data: " << std::string(mLastPos, static_cast<size_t>(j));
+                        JSON_CHUNK_PROCESSING
+                            << "JSON error callback."
+                            << " Data: " << std::string(mLastPos, static_cast<size_t>(j));
 
                         auto& callback = filterit->second;
-                        if (!callback(&jsonData))
+                        auto result = callback(&jsonData);
+                        if (result != CallbackResult::SUCCESS)
                         {
                             LOG_err << "Parsing error processing error streaming filter"
                                     << " Data: " << std::string(mLastPos, static_cast<size_t>(j));
@@ -1413,10 +1432,11 @@ m_off_t JSONSplitter::processChunk(std::map<string, std::function<bool (JSON *)>
     mProcessedBytes = mPos - mLastPos;
     m_off_t consumedBytes = mLastPos - data;
 
-    JSON_verbose << "JSON leaving processChunk at path " << mCurrentPath << "."
-                 << " Data: " << std::string(mPos, strlen(mPos) < 32 ? strlen(mPos) : 32)
-                 << " Processed: " << mProcessedBytes << " Consumed: " << consumedBytes
-                 << " Next start: " << std::string(mLastPos, strlen(mLastPos) < 32 ? strlen(mLastPos) : 32);
+    JSON_CHUNK_PROCESSING << "JSON leaving processChunk at path " << mCurrentPath << "."
+                          << " Data: " << std::string(mPos, strlen(mPos) < 32 ? strlen(mPos) : 32)
+                          << " Processed: " << mProcessedBytes << " Consumed: " << consumedBytes
+                          << " Next start: "
+                          << std::string(mLastPos, strlen(mLastPos) < 32 ? strlen(mLastPos) : 32);
 
     return consumedBytes;
 }
@@ -1439,7 +1459,7 @@ bool JSONSplitter::isStarting()
 int JSONSplitter::strEnd()
 {
     const char* ptr = mPos;
-    while ((ptr = strchr(ptr + 1, '"')))
+    while ((ptr = strchr(ptr + 1, '"')) != nullptr)
     {
         const char *e = ptr;
         while (*(--e) == '\\')
@@ -1472,7 +1492,7 @@ int JSONSplitter::numEnd()
     return -1;
 }
 
-void JSONSplitter::parseError(std::map<string, std::function<bool (JSON *)> > *filters)
+void JSONSplitter::parseError(std::map<string, FilterCallback>* filters)
 {
     if (filters)
     {
@@ -1493,14 +1513,15 @@ void JSONSplitter::parseError(std::map<string, std::function<bool (JSON *)> > *f
     assert(false);
 }
 
-bool JSONSplitter::chunkProcessingFinishedSuccessfully(std::map<std::string, std::function<bool(JSON*)>>* filters)
+bool JSONSplitter::chunkProcessingFinishedSuccessfully(
+    std::map<std::string, FilterCallback>* filters)
 {
     auto filterit = filters->find(">");
     if (filterit != filters->end())
     {
         JSON jsonData("");
         auto& callback = filterit->second;
-        if (!callback(&jsonData))
+        if (CallbackResult::SUCCESS != callback(&jsonData))
         {
             return false;
         }
