@@ -87,7 +87,8 @@ void MediaFileInfo::requestCodecMappingsOneTime(MegaClient* client, const LocalP
         }
 
         LOG_debug << "Requesting code mappings";
-        client->reqs.add(new CommandMediaCodecs(client, &MediaFileInfo::onCodecMappingsReceiptStatic));
+        client->queueCommand(
+            new CommandMediaCodecs(client, &MediaFileInfo::onCodecMappingsReceiptStatic));
         mediaCodecsRequested = true;
     }
 }
@@ -917,7 +918,7 @@ void MediaProperties::extractMediaPropertyFileAttributes(LocalPath& localFilenam
         }
         catch (std::exception& e)
         {
-            LOG_err << "exception caught reading media file attibutes: " << e.what();
+            LOG_err << "exception caught reading media file attributes: " << e.what();
         }
         catch (...)
         {
