@@ -38,9 +38,7 @@ Process::EnvironmentChanger::EnvironmentChanger(const unordered_map<string, stri
 {
     for (auto& i : env)
     {
-        bool found = false;
-        string val = Utils::getenv(i.first, &found);
-        if (found) 
+        if (const auto [val, hasValue] = Utils::getenv(i.first); hasValue)
         {
             saved[i.first] = val;
         }
@@ -197,27 +195,31 @@ bool Process::run(const vector<string>& args, const unordered_map<string, string
     // Child process - specific code
     if (childPid == 0)
     {
+        // These belong to the parent now.
+        ::close(readFd);
+        ::close(readErrorFd);
+
         // Redirect streams to the parent process
         // stdout
         ::close(STDOUT_FILENO);
         if (dup2(childReadFd, STDOUT_FILENO) == -1)
         {
             reportError("Could not redirect stdout");
-            exit(1);
+            _exit(EXIT_FAILURE);
         }
         // stderr
         ::close(STDERR_FILENO);
         if (dup2(childReadErrorFd, STDERR_FILENO) == -1)
         {
             reportError("Could not redirect stderr");
-            exit(1);
+            _exit(EXIT_FAILURE);
         }
         
         // Prepare command-line arguments for child process
         if (args.empty())
         {
             cerr << "Process: Can not execute, no arguments given" << endl;
-            exit(1);
+            _exit(EXIT_FAILURE);
         }
         vector<char*> argv;
         for (vector<string>::const_iterator i = args.begin(); i != args.end(); ++i)
@@ -235,10 +237,14 @@ bool Process::run(const vector<string>& args, const unordered_map<string, string
         // cerr so parent process sees this
         cerr << "Could not execute '" + string(argv[0]) + "'" << ": " << savedErrno << ": " << strerror(savedErrno) << endl;
         reportError("Could not execute '" + string(argv[0]) + "'", savedErrno);
-        exit(1);
+        _exit(EXIT_FAILURE);
     }
 //    else --> parent process
 
+    // These belong to the child now.
+    ::close(childReadFd);
+    ::close(childReadErrorFd);
+
     // Set the read/write from/to child process streams as non-blocking
     // stdout of child process
     if (fcntl(readFd, F_SETFL, fcntl(readFd, F_GETFL) | O_NONBLOCK) == -1)
@@ -609,7 +615,7 @@ bool Process::checkStatus()
         }
         else
         {
-            assert(!"waitpid() but not exited not signalled");
+            assert(false && "waitpid() but not exited not signalled");
             setWaitFailureStatus(); // otherwise may spin forever
         }
         return true;
@@ -728,9 +734,9 @@ void ConsoleProgressBar::show() const
         std::cout << '\r';
 }
 
-void ConsoleProgressBar::setPrefix(const std::string &value)
+void ConsoleProgressBar::setPrefix(const std::string& newPrefix)
 {
-    prefix = value;
+    prefix = newPrefix;
 }
 
 ConsoleProgressBar::ConsoleProgressBar(size_t imax, bool writeNewLine)
@@ -758,7 +764,7 @@ std::ostream& ConsoleProgressBar::put(std::ostream& os) const {
 
     m_time_t elapsed = m_time() - start;
     size_t totalSec = size_t((double)elapsed / ((double)value / (double)max));
-    m_time_t etta = totalSec - elapsed;
+    m_time_t etta = static_cast<m_time_t>(totalSec) - elapsed;
 
     char ettaStr[1024];
     tm tm;
