@@ -20,12 +20,17 @@
  */
 
 #include "mega/request.h"
+
 #include "mega/command.h"
+#include "mega/common/badge.h"
 #include "mega/logging.h"
 #include "mega/megaclient.h"
 
 namespace mega {
 
+// Convenience.
+using common::Badge;
+
 bool Request::isFetchNodes() const
 {
     return cmds.size() == 1 && dynamic_cast<CommandFetchNodes*>(cmds.back().get());
@@ -245,12 +250,13 @@ void Request::process(MegaClient* client)
 
         if (cmd->mSeqtagArray && processingJson.enterarray())
         {
-            // Some commands need to return seqtag and also some JSON,
-            // in which case they are in an array with `st` first, and the JSON second
-            // Some commands might or might not produce `st`.  And might return a string.
-            // So in the case of success with a string return, but no `st`, the array is [0, "returnValue"]
+            // Responses parsed here:
+            //      -> ["<st>", <json>] : Sequence Tag + a json
+            //      -> [0, "string"] : API_OK + a no-json-string
+            // Commands should be marked as mSeqtagArray=true to be differentiated from a json array
+            // like ["ES","FR","IE","GB"] If a command returning a string fails, there is no array,
+            // just the error code
             // If the command failed, there is no array, just the error code
-            assert(cmd->mV3);
             assert(*processingJson.pos == '0' || *processingJson.pos == '\"');
             if (*processingJson.pos == '0' && *(processingJson.pos+1) == ',')
             {
@@ -270,9 +276,10 @@ void Request::process(MegaClient* client)
                 parsedOk = false;
             }
         }
-        else if (mV3 && *processingJson.pos == '"')
+        else if (*processingJson.pos == '"')
         {
-            // For v3 commands, a string result is a string which is a seqtag.
+            // Responses parsed here:
+            //      -> "<st>" : Only a Sequence Tag.
             if (!processSeqTag(cmd, false, parsedOk, false, processingJson))
             {
                 // we need to wait for sc processing to catch up with the seqtag we just read
@@ -282,7 +289,9 @@ void Request::process(MegaClient* client)
         }
         else
         {
-            // straightforward case - plain JSON response, no seqtag
+            // Responses parsed here:
+            //      -> 0 or -N : Error codes, plain numeric values.
+            //      -> {..} or [..,..,] : Json response, could be an object or an array.
             parsedOk = processCmdJSON(cmd, true, processingJson);
         }
 
@@ -370,7 +379,6 @@ void Request::swap(Request& r)
 {
     // we use swap to move between queues, but process only after it gets into the completedreqs
     cmds.swap(r.cmds);
-    std::swap(mV3, r.mV3);
 
     std::swap(cachedJSON, r.cachedJSON);
     std::swap(cachedIdempotenceId, r.cachedIdempotenceId);
@@ -403,7 +411,7 @@ void RequestDispatcher::sendDeferred()
 }
 #endif
 
-void RequestDispatcher::add(Command *c)
+void RequestDispatcher::add(Badge<MegaClient>, Command* c)
 {
 #if defined(MEGA_MEASURE_CODE) || defined(DEBUG)
     if (deferRequests && deferRequests(c))
@@ -425,16 +433,6 @@ void RequestDispatcher::add(Command *c)
         nextreqs.push_back(Request());
     }
 
-    if (!nextreqs.back().empty() && nextreqs.back().mV3 != c->mV3)
-    {
-        LOG_debug << "Starting an additional Request for v3 transition " << c->mV3;
-        nextreqs.push_back(Request());
-    }
-    if (nextreqs.back().empty())
-    {
-        nextreqs.back().mV3 = c->mV3;
-    }
-
     nextreqs.back().add(c);
     if (c->batchSeparately)
     {
@@ -458,8 +456,16 @@ bool RequestDispatcher::readyToSend() const
 
 bool RequestDispatcher::cmdsInflight() const
 {
-    // stays true even through network errors, -3, retries, etc until we get that response
-    return !inflightreq.empty();
+    // stays true even through network errors, retries, etc until we get that response
+    // but not when the API has notified that commands were not applied
+    return !inflightreq.empty() && inflightFailReason != RETRY_API_LOCK &&
+           inflightFailReason != RETRY_RATE_LIMIT;
+}
+
+bool RequestDispatcher::retryReasonIsApi() const
+{
+    return !inflightreq.empty() &&
+           (inflightFailReason == RETRY_API_LOCK || inflightFailReason == RETRY_RATE_LIMIT);
 }
 
 Command* RequestDispatcher::getCurrentCommand(bool currSeqtagSeen)
@@ -467,7 +473,9 @@ Command* RequestDispatcher::getCurrentCommand(bool currSeqtagSeen)
     return currSeqtagSeen ? inflightreq.getCurrentCommand() : nullptr;
 }
 
-string RequestDispatcher::serverrequest(bool &includesFetchingNodes, bool& v3, MegaClient* client, string& idempotenceId)
+string RequestDispatcher::serverrequest(bool& includesFetchingNodes,
+                                        MegaClient* client,
+                                        string& idempotenceId)
 {
     if (!inflightreq.empty() && inflightFailReason != RETRY_NONE)
     {
@@ -487,7 +495,6 @@ string RequestDispatcher::serverrequest(bool &includesFetchingNodes, bool& v3, M
     }
     string requestJSON = inflightreq.get(client, reqid, idempotenceId);
     includesFetchingNodes = inflightreq.isFetchNodes();
-    v3 = inflightreq.mV3;
 #ifdef MEGA_MEASURE_CODE
     csRequestsSent += inflightreq.size();
     csBatchesSent += 1;
