@@ -58,7 +58,7 @@ void TreeProcDel::proc(MegaClient* client, std::shared_ptr<Node> n)
     client->mNodeManager.notifyNode(n);
     handle userHandle = ISUNDEF(mOriginatingUser) ? n->owner : mOriginatingUser;
 
-    if (userHandle != client->me)
+    if (userHandle != client->me && !client->loggedIntoFolder())
     {
         client->useralerts.noteSharedNode(userHandle, n->type, 0, n.get());
     }
@@ -93,48 +93,15 @@ void TreeProcCopy::proc(MegaClient* client, std::shared_ptr<mega::Node> n)
 {
     if (allocated)
     {
-        string attrstring;
-        SymmCipher key;
-        assert(nc > 0);
-        NewNode* t = &nn[--nc];
-
-        // copy node
-        t->source = NEW_NODE;
-        t->type = n->type;
-        t->nodehandle = n->nodehandle;
-        t->parenthandle = n->parent ? n->parent->nodehandle : UNDEF;
-
-        // copy key (if file) or generate new key (if folder)
-        if (n->type == FILENODE) t->nodekey = n->nodekey();
-        else
-        {
-            byte buf[FOLDERNODEKEYLENGTH];
-            client->rng.genblock(buf,sizeof buf);
-            t->nodekey.assign((char*)buf,FOLDERNODEKEYLENGTH);
-        }
-
-        t->attrstring.reset(new string);
-        if(t->nodekey.size())
-        {
-            key.setkey((const byte*)t->nodekey.data(),n->type);
-
-            AttrMap tattrs;
-            tattrs.map = n->attrs.map;
-            nameid rrname = AttrMap::string2nameid("rr");
-            attr_map::iterator it = tattrs.map.find(rrname);
-            if (it != tattrs.map.end())
-            {
-                LOG_debug << "Removing rr attribute";
-                tattrs.map.erase(it);
-            }
-            if (resetSensitive && tattrs.map.erase(AttrMap::string2nameid("sen")))
-            {
-                LOG_debug << "Removing sen attribute";
-            }
-
-            tattrs.getjson(&attrstring);
-            client->makeattr(&key, t->attrstring, attrstring.c_str());
-        }
+        client->putnodes_prepareCopy(nn,
+                                     nc,
+                                     n->type,
+                                     n->nodehandle,
+                                     n->parent ? n->parent->nodehandle : UNDEF,
+                                     n->nodekey(),
+                                     n->attrs,
+                                     resetSensitive,
+                                     false);
     }
     else nc++;
 }
