@@ -23,6 +23,7 @@
 #include "../include/megaapi_impl.h"
 #include "gtest/gtest.h"
 #include "mega.h"
+#include "mega/scoped_helpers.h"
 #include "sdk_test_data_provider.h"
 #include "test.h"
 
@@ -51,7 +52,9 @@ static const unsigned int maxTimeout    = 600;      // Maximum time (seconds) to
 static const unsigned int defaultTimeout = 60;      // Normal time for most operations (seconds) to wait for response from server
 static const unsigned int defaultTimeoutMs = defaultTimeout * 1000;
 static const unsigned int waitForSyncsMs = 4000;    // Time to wait after a sync has been created and before adding new files to it
+constexpr unsigned int cleanupCatchupTimeoutSecs = 15;
 
+#ifdef ENABLE_SYNC
 /**
  * @brief Wrapper struct of MegaListener to get all information related to a MEGA account
  *  - make sure callbacks are consistent - added() first, nothing after deleted(), etc.
@@ -206,6 +209,7 @@ struct MegaListenerDeregisterer
         api->removeListener(listener);
     }
 };
+#endif
 
 struct TransferTracker : public ::mega::MegaTransferListener
 {
@@ -219,6 +223,8 @@ struct TransferTracker : public ::mega::MegaTransferListener
 
     bool mTempFileRemoved{false};
     MegaHandle resultNodeHandle = UNDEF;
+    m_off_t mTransferSpeed{-1};
+    m_off_t mTransferMeanSpeed{-1};
 
     TransferTracker(MegaApi *api): mApi(api), futureResult(promiseResult.get_future())
     {
@@ -234,9 +240,11 @@ struct TransferTracker : public ::mega::MegaTransferListener
         }
     }
 
-    void onTransferStart(MegaApi*, MegaTransfer*) override
+    void onTransferStart(MegaApi*, MegaTransfer* transfer) override
     {
         // called back on a different thread
+        LOG_debug << "TransferTracker::onTransferStart callback received -> set started true for "
+                  << (transfer && transfer->getFileName() ? transfer->getFileName() : "<null>");
         started = true;
     }
 
@@ -247,6 +255,10 @@ struct TransferTracker : public ::mega::MegaTransferListener
 
         // called back on a different thread
         resultNodeHandle = transfer->getNodeHandle();
+        mTransferSpeed = transfer->getSpeed();
+        mTransferMeanSpeed = transfer->getMeanSpeed();
+        LOG_debug << "TransferTracker::onTransferFinish - tSpeed = " << mTransferSpeed
+                  << ", tMeanSpeed = " << mTransferMeanSpeed;
         result = static_cast<ErrorCodes>(error->getErrorCode());
         finished = true;
 
@@ -366,6 +378,16 @@ struct RequestTracker : public ::mega::MegaRequestListener
         return request ? request->getFlag() : false;
     }
 
+    int getParamType()
+    {
+        return request ? request->getParamType() : -999;
+    }
+
+    int getNumber()
+    {
+        return request ? static_cast<int>(request->getNumber()) : -999;
+    }
+
     template<typename... Args, typename... Params>
     static unique_ptr<RequestTracker> async(MegaApi& api,
                                             void (MegaApi::*mf)(Params...),
@@ -465,8 +487,13 @@ Error result(const Expected<T>& expected)
 template<typename T, typename = std::enable_if_t<IsExpectedV<RemoveCVRefT<T>>>>
 decltype(auto) value(T&& expected)
 {
-    assert(result(expected) == API_OK);
-
+#ifndef NDEBUG
+    if (auto res = result(expected); res != API_OK)
+    {
+        LOG_err << "value: unexpected result: " << res;
+        assert(res == API_OK && "value: unexpected result");
+    }
+#endif
     return std::get<1>(std::forward<T>(expected));
 }
 
@@ -475,7 +502,6 @@ using MegaApiTestPointer = std::unique_ptr<MegaApiTest, MegaApiTestDeleter>;
 // Fixture class with common code for most of tests
 class SdkTest:
     public SdkTestBase,
-    public SdkTestDataProvider,
     public MegaListener,
     public MegaRequestListener,
     MegaTransferListener,
@@ -628,13 +654,25 @@ public:
         shared_ptr<const MegaStringTable> stringTable;
     };
 
+    constexpr static unsigned MAX_VAULT_CHILDREN = 2;
     std::vector<PerApi> mApi;
     std::vector<MegaApiTestPointer> megaApi;
+    std::vector<ScopedDestructor> mAccountsRestorer;
 
+    std::function<void(MegaTransfer*)> onTransferStartCustomCb;
     m_off_t onTransferStart_progress;
     m_off_t onTransferUpdate_progress;
     m_off_t onTransferUpdate_filesize;
     unsigned onTranferFinishedCount = 0;
+    bool mCleanupSuccess{true};
+
+    void updateCleanupStatus(const bool stageSucceeded)
+    {
+        if (!mCleanupSuccess || stageSucceeded)
+            return;
+
+        mCleanupSuccess = stageSucceeded;
+    }
 
     struct SdkTestTransferStats
     {
@@ -654,10 +692,13 @@ public:
     SdkTestTransferStats onTransferFinish_transferStats{};
 
 protected:
+#define CASE_info LOG_info << getLogPrefix()
+
     void SetUp() override;
     void TearDown() override;
 
     void Cleanup();
+    void setTestAccountsToFree(unsigned int nApi);
 
     int getApiIndex(MegaApi* api);
     bool getApiIndex(MegaApi* api, size_t& apindex);
@@ -667,9 +708,17 @@ protected:
 
     void testPrefs(const std::string& title, int type);
     void testRecents(const std::string& title, bool useSensitiveExclusion);
+    void testCloudRaidTransferResume(const bool fromNonRaid, const std::string& logPre);
+    void testResumableTrasfers(const std::string& data, const size_t timeoutInSecs);
+    void testHashcash(const bool logoutDuringLogin);
 
+    void printCleanupErrMsg(const string& prefix,
+                            const string& errDetails,
+                            const unsigned accountIdx,
+                            const int errCode,
+                            const bool localCleanupSuccess) const;
 #ifdef ENABLE_CHAT
-    void cancelSchedMeetings();
+    void cleanupSchedMeetings(const unsigned nApi);
 #endif
 
     void syncTestEnsureMyBackupsRemoteFolderExists(unsigned apiIdx);
@@ -705,14 +754,19 @@ protected:
     void onSyncRemoteRootChanged(MegaApi*, MegaSync*) override {}
 
     void onGlobalSyncStateChanged(MegaApi*) override {}
+
+    void cleanupSyncs(const unsigned int nApi);
     void purgeVaultTree(unsigned int apiIndex, MegaNode *vault);
 #endif
 #ifdef ENABLE_CHAT
     void onChatsUpdate(MegaApi *api, MegaTextChatList *chats) override;
+    void cleanupChatLinks(const unsigned int nApi, std::set<MegaHandle>& skipChats);
+    void cleanupChatrooms(const unsigned int nApi);
 #endif
     void onEvent(MegaApi* api, MegaEvent *event) override;
 
     void resetOnNodeUpdateCompletionCBs();
+    void cleanupCatchupWithApi(const unsigned int apiIndex, const unsigned int timeoutSecs);
 
     onNodesUpdateCompletion_t createOnNodesUpdateLambda(const MegaHandle&, int, bool& flag);
 public:
@@ -725,6 +779,12 @@ public:
     void resumeSession(const char *session, unsigned apiIndex = 0);
 
     void purgeTree(unsigned int apiIndex, MegaNode *p, bool depthfirst = true);
+    void cleanupContacts(const unsigned int nApi);
+    void cleanupShares(const unsigned int nApi);
+    void cleanupNodeLinks(const unsigned int nApi);
+    void cleanupNodes(const unsigned int nApi);
+    void cleanupContactRequests(const unsigned int nApi);
+    void cleanupLocalFiles();
 
     bool waitForResponse(bool *responseReceived, unsigned int timeout = maxTimeout);
 
@@ -733,20 +793,50 @@ public:
     static bool WaitFor(const std::function<bool()>& predicate, unsigned timeoutMs);
 
     bool synchronousRequest(unsigned apiIndex, int type, std::function<void()> f, unsigned int timeout = maxTimeout);
+    bool synchronousRequestIgnoreErr(unsigned apiIndex,
+                                     int type,
+                                     std::function<void()> f,
+                                     unsigned int timeout = maxTimeout);
     bool synchronousTransfer(unsigned apiIndex, int type, std::function<void()> f, unsigned int timeout = maxTimeout);
 
     // *** WARNING *** THESE FUNCTIONS RETURN VALUE ARE SUBJECT TO RACE CONDITIONS
     // convenience functions - template args just make it easy to code, no need to copy all the exact argument types with listener defaults etc. To add a new one, just copy a line and change the flag and the function called.
     // WARNING: any sort of race can result in the lastError being set from some other command - better to use the listener based ones in the next list below
     template<typename... Args>
-    int synchronousCatchup(unsigned apiIndex, Args... args)
+    int synchronousCatchup(unsigned apiIndex, unsigned int timeoutSecs, Args... args)
     {
-        synchronousRequest(apiIndex,
-                           MegaRequest::TYPE_CATCHUP,
-                           [this, apiIndex, args...]()
-                           {
-                               megaApi[apiIndex]->catchup(args...);
-                           });
+        if (megaApi[apiIndex]->isEphemeralPlusPlus())
+        {
+            return API_OK;
+        }
+
+        synchronousRequest(
+            apiIndex,
+            MegaRequest::TYPE_CATCHUP,
+            [this, apiIndex, args...]()
+            {
+                megaApi[apiIndex]->catchup(args...);
+            },
+            timeoutSecs);
+        return mApi[apiIndex].lastError;
+    }
+
+    template<typename... Args>
+    int synchronousCatchupIgnoreErr(unsigned apiIndex, unsigned int timeoutSecs, Args... args)
+    {
+        if (megaApi[apiIndex]->isEphemeralPlusPlus())
+        {
+            return API_OK;
+        }
+
+        synchronousRequestIgnoreErr(
+            apiIndex,
+            MegaRequest::TYPE_CATCHUP,
+            [this, apiIndex, args...]()
+            {
+                megaApi[apiIndex]->catchup(args...);
+            },
+            timeoutSecs);
         return mApi[apiIndex].lastError;
     }
 
@@ -949,6 +1039,15 @@ public:
         return rt;
     }
 
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncSetUnshareableNodeCoordinates(unsigned int apiIndex,
+                                                                       requestArgs... args)
+    {
+        auto requestTracker{std::make_unique<RequestTracker>(megaApi[apiIndex].get())};
+        megaApi[apiIndex]->setUnshareableNodeCoordinates(args..., requestTracker.get());
+        return requestTracker;
+    }
+
     template<typename... requestArgs>
     int doGetDeviceName(unsigned apiIndex, string* dvc, requestArgs... args)
     {
@@ -1003,7 +1102,29 @@ public:
         return rt.waitForResult();
     }
 
-    template<typename ... requestArgs> int doStartUpload(unsigned apiIndex, MegaHandle* newNodeHandleResult, requestArgs... args) { TransferTracker tt(megaApi[apiIndex].get()); megaApi[apiIndex]->startUpload(args..., &tt); auto e = tt.waitForResult(); if (newNodeHandleResult) *newNodeHandleResult = tt.resultNodeHandle; return e;}
+    template<typename... requestArgs>
+    int doStartUpload(unsigned apiIndex, MegaHandle* newNodeHandleResult, requestArgs... args)
+    {
+        TransferTracker tt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->startUpload(args..., &tt);
+        auto e = tt.waitForResult();
+        if (newNodeHandleResult)
+            *newNodeHandleResult = tt.resultNodeHandle;
+        return e;
+    }
+
+    template<typename... requestArgs>
+    std::tuple<int, m_off_t, m_off_t> doStartUploadWithSpeed(unsigned apiIndex,
+                                                             MegaHandle* newNodeHandleResult,
+                                                             requestArgs... args)
+    {
+        TransferTracker tt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->startUpload(args..., &tt);
+        auto e = tt.waitForResult();
+        if (newNodeHandleResult)
+            *newNodeHandleResult = tt.resultNodeHandle;
+        return {e, tt.mTransferSpeed, tt.mTransferMeanSpeed};
+    }
     template<typename ... requestArgs> int doStartDownload(unsigned apiIndex, requestArgs... args) { TransferTracker tt(megaApi[apiIndex].get()); megaApi[apiIndex]->startDownload(args..., &tt); auto e = tt.waitForResult(); return e;}
     template<typename ... requestArgs> int doSetFileVersionsOption(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->setFileVersionsOption(args..., &rt); return rt.waitForResult(); }
     template<typename ... requestArgs> int doRemoveVersion(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->removeVersion(args..., &rt); return rt.waitForResult(); }
@@ -1148,9 +1269,10 @@ public:
     void deleteFile(string filename);
     void deleteFolder(string foldername);
 
-    void fetchNodesForAccounts(const unsigned howMany);
-    void getAccountsForTest(unsigned howMany = 1,
-                            bool fetchNodes = true,
+    // peform fetchnodes for all test involved accounts sequentially to avoid API locks
+    void fetchNodesForAccountsSequentially(const unsigned howMany);
+    void getAccountsForTest(const unsigned howMany = 1,
+                            const bool fetchNodes = true,
                             const int clientType = MegaApi::CLIENT_TYPE_DEFAULT);
     void configureTestInstance(unsigned index,
                                const std::string& email,
@@ -1182,6 +1304,10 @@ public:
 
     void getContactRequest(unsigned int apiIndex, bool outgoing, int expectedSize = 1);
 
+    std::pair<int, MegaHandle> createRemoteFolder(const unsigned int apiIndex,
+                                                  const char* name,
+                                                  MegaNode* parent,
+                                                  const int timeout = maxTimeout);
     MegaHandle createFolder(unsigned int apiIndex, const char *name, MegaNode *parent, int timeout = maxTimeout);
 
     // SMS verification was deprecated. This function should be removed in the future,
@@ -1194,7 +1320,10 @@ public:
     void synchronousMediaUploadIncomplete(unsigned int apiIndex, int64_t fileSize, const char* filename, const char* fileEncrypted, std::string& fingerprint, std::string& string64UploadToken, std::string& string64FileKey);
 
 #ifdef ENABLE_CHAT
-    void createChat(bool group, MegaTextChatPeerList *peers, int timeout = maxTimeout);
+    void createChat(bool group,
+                    MegaTextChatPeerList* peers,
+                    bool isPublicChat = false,
+                    int timeout = maxTimeout);
 
     /**
      * @brief Creates a chat room from the mApi[creatorIndex] account waiting for all the events to
@@ -1213,7 +1342,14 @@ public:
     MegaHandle createChatWithChecks(const unsigned int creatorIndex,
                                     const std::vector<unsigned int>& invitedIndices,
                                     const bool group,
+                                    const bool isPublicChat,
                                     const unsigned int timeout_sec = maxTimeout);
+
+    MegaHandle getCommander();
+
+    void testChat(bool isPublicChat);
+
+    void testGiveRemoveChatAccess(bool isPublicChat);
 #endif
 
     template<typename... requestArgs>
@@ -1224,6 +1360,34 @@ public:
         return rt.waitForResult();
     }
 
+    template<typename... requestArgs>
+    ErrorCodes doGetMaxUploadConnections(const unsigned apiIndex,
+                                         int& direction,
+                                         int& maxConnections,
+                                         requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->getMaxUploadConnections(args..., &rt);
+        const auto err = rt.waitForResult();
+        direction = err == API_OK ? rt.getParamType() : -999;
+        maxConnections = err == API_OK ? rt.getNumber() : -999;
+        return err;
+    }
+
+    template<typename... requestArgs>
+    ErrorCodes doGetMaxDownloadConnections(const unsigned apiIndex,
+                                           int& direction,
+                                           int& maxConnections,
+                                           requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->getMaxUploadConnections(args..., &rt);
+        const auto err = rt.waitForResult();
+        direction = err == API_OK ? rt.getParamType() : -999;
+        maxConnections = err == API_OK ? rt.getNumber() : -999;
+        return err;
+    }
+
     /* MegaVpnCredentials */
     template<typename... requestArgs>
     int doGetVpnRegions(unsigned apiIndex,
@@ -1497,6 +1661,25 @@ public:
      * @example calling this inside `TEST_F(SdkTest, Foo)` returns `"SdkTest_Foo_"`
      */
     std::string getFilePrefix() const;
+
+    /**
+     * @brief Set value for ^!devopt user attribute
+     *
+     * Format is a json, per example:
+     * {"utqamo":1}
+     *
+     * @param value
+     * @param index MegaApi instance index
+     */
+    void setDevOptUserAttribute(const std::string& value, uint32_t index) const;
+
+    using GetDevOptAttrResult = std::pair<int, std::optional<std::string>>;
+    /**
+     * @brief Get value for ^!devopt user attribute
+     * @param index MegaApi instance index
+     * @return {error, attribute value}
+     */
+    GetDevOptAttrResult getDevOptUserAttribute(uint32_t index) const;
 };
 
 /**
@@ -1511,6 +1694,21 @@ public:
  */
 auto accountLevelRestorer(MegaApi& client) -> ScopedDestructor;
 
+/**
+ * @brief
+ * Return an object that will restore client's plan on destruction.
+ *
+ * @param clients
+ * Vector of pointers to valid/active MegaApi instances to perform the restoration
+ *
+ * @param idx
+ * Index of the MegaApi instance to use at restoration execution
+ *
+ * @return
+ * An object that will restore client's plan on destruction.
+ */
+ScopedDestructor accountLevelRestorer(std::vector<MegaApiTestPointer>& clients, unsigned int idx);
+
 /**
  * @brief
  * Create a directory with a given name under a specified parent.
@@ -1542,7 +1740,19 @@ auto createDirectory(MegaApi& client, const MegaNode& parent, const std::string&
  * An account level restorer on success.
  * An error on failure.
  */
-auto elevateToPro(MegaApi& client) -> Expected<ScopedDestructor>;
+auto scopedToPro(MegaApi& client) -> Expected<ScopedDestructor>;
+
+/**
+ * @brief
+ * Demote client to a free plan.
+ *
+ * @param client
+ * The client who we want to demote to a free plan.
+ *
+ * @return
+ * An error on failure or succuss.
+ */
+auto demoteToFree(MegaApi& client) -> Error;
 
 /**
  * @brief
