@@ -21,7 +21,8 @@
 
 namespace sdk_test
 {
-
+using namespace std::chrono_literals;
+static constexpr auto MAX_TIMEOUT = 3min; // Timeout for operations in this file
 #ifdef ENABLE_SYNC
 
 /**
@@ -49,6 +50,11 @@ std::unique_ptr<::mega::MegaSync> waitForSyncState(::mega::MegaApi* megaApi,
                                                    ::mega::MegaSync::SyncRunningState runState,
                                                    ::mega::MegaSync::Error err);
 
+/**
+ * @brief Waits for the sync stall state to be set certain amount of time (10 seconds).
+ */
+bool waitForSyncStallState(::mega::MegaApi* const megaApi);
+
 /**
  * @brief Synchronously start a TWO_WAY sync between the given local path and the remote node with
  * the given handle.
@@ -115,6 +121,11 @@ bool suspendSync(::mega::MegaApi* megaApi, const ::mega::handle backupID);
  */
 bool disableSync(::mega::MegaApi* megaApi, const ::mega::handle backupID);
 
+/**
+ * @brief Check MegaApi flags for sync stall state.
+ */
+bool isSyncStalled(::mega::MegaApi* const megaApi);
+
 /**
  * @brief Get a vector with all the reported stalls.
  */
@@ -122,6 +133,16 @@ std::vector<std::unique_ptr<::mega::MegaSyncStall>> getStalls(::mega::MegaApi* m
 
 #endif
 
+/**
+ * @brief Get a pair formed by a vector with the names of the nodes that are children of the node
+ * with the given handle and a unique ptr to MegaNodeList.
+ *
+ * If any of the operations to get the nodes fails, a pair formed by nullopt and nullptr is
+ * returned.
+ */
+std::pair<std::optional<std::vector<std::string>>, std::unique_ptr<::mega::MegaNodeList>>
+    getCloudFirstChildren(::mega::MegaApi* megaApi, const ::mega::MegaHandle nodeHandle);
+
 /**
  * @brief Get a vector with the names of the nodes that are children of the node with the given
  * handle.
@@ -159,6 +180,31 @@ bool checkAndExpectThat(const T& value, const MatcherT& matcher)
     return matched;
 }
 
+using TransferFinishCallback =
+    std::function<void(::mega::MegaApi*, ::mega::MegaTransfer*, ::mega::MegaError*)>;
+
+/**
+ * @brief Downloads a file from MEGA
+ * @see MegaApi::startDownload for more details
+ * @return a Numeric errCode corresponding to MegaError received at
+ * MegaTransferListener::onTransferFinish, or nullopt if onTransferFinish is never called.
+ * Note: If onTransferFinish is called but MegaError is not valid API_EINTERNAL will be returned.
+ */
+std::optional<int> downloadNode(
+    ::mega::MegaApi* megaApi,
+    ::mega::MegaNode* node,
+    const std::filesystem::path& fsPath,
+    bool pathIsFolder,
+    const std::chrono::seconds timeoutInSecs = MAX_TIMEOUT,
+    const int collisionCheck = ::mega::MegaTransfer::COLLISION_CHECK_FINGERPRINT,
+    const int collisionResolution = ::mega::MegaTransfer::COLLISION_RESOLUTION_NEW_WITH_N,
+    TransferFinishCallback transferFinishCallback = nullptr,
+    const char* customName = nullptr,
+    const char* appData = nullptr,
+    const bool startFirst = false,
+    ::mega::MegaCancelToken* cancelToken = nullptr,
+    const bool undelete = false);
+
 /**
  * @brief Uploads the file in the given path to the given parentNode.
  */
@@ -177,6 +223,11 @@ std::unique_ptr<::mega::MegaNode> uploadFile(::mega::MegaApi* megaApi,
                                   const std::string& name,
                                   const ::mega::MegaNode::PasswordNodeData* data,
                                   const ::mega::handle parentNodeHandle);
+
+::mega::handle createCreditCardNode(::mega::MegaApi* megaApi,
+                                    const std::string& name,
+                                    const ::mega::MegaNode::CreditCardNodeData* data,
+                                    const ::mega::handle parentNodeHandle);
 }
 
 #endif // INCLUDE_INTEGRATION_INTEGRATION_TEST_UTILS_H_
