@@ -150,6 +150,32 @@ private:
     }
 };
 
+/**
+ * @class MockNodesUpdateListener
+ * @brief A simple mock listener to track nodes updates
+ */
+class MockNodesUpdateListener: public ::mega::MegaListener
+{
+public:
+    MockNodesUpdateListener(::mega::MegaApi* megaApi = nullptr):
+        mMegaApi{megaApi}
+    {}
+
+    MOCK_METHOD(void,
+                onNodesUpdate,
+                (::mega::MegaApi * api, ::mega::MegaNodeList* nodes),
+                (override));
+
+    ~MockNodesUpdateListener()
+    {
+        if (mMegaApi)
+            mMegaApi->removeListener(this);
+    }
+
+private:
+    ::mega::MegaApi* mMegaApi{nullptr};
+};
+
 /**
  * @class MockSyncListener
  * @brief Mock listener only implementing methods for transfers
@@ -256,6 +282,16 @@ private:
 class MockSyncListener: public ::mega::MegaListener
 {
 public:
+    MockSyncListener(::mega::MegaApi* megaApi = nullptr):
+        mMegaApi{megaApi}
+    {}
+
+    ~MockSyncListener()
+    {
+        if (mMegaApi)
+            mMegaApi->removeListener(this);
+    }
+
     MOCK_METHOD(
         void,
         onSyncFileStateChanged,
@@ -280,6 +316,9 @@ public:
                 onRequestFinish,
                 (::mega::MegaApi*, ::mega::MegaRequest*, ::mega::MegaError*),
                 (override));
+
+private:
+    ::mega::MegaApi* mMegaApi{nullptr};
 };
 #endif // ENABLE_SYNC
 
