@@ -13,6 +13,13 @@ void SdkTestPasswordManager::SetUp()
     ASSERT_NE(megaApi[0], nullptr);
     mApi = megaApi[0].get();
     ASSERT_NO_FATAL_FAILURE(initPasswordManagerBase());
+    ASSERT_NO_FATAL_FAILURE(removePWMNodes());
+}
+
+void SdkTestPasswordManager::TearDown()
+{
+    removePWMNodes();
+    SdkTest::TearDown();
 }
 
 void SdkTestPasswordManager::initPasswordManagerBase()
@@ -31,3 +38,10 @@ void SdkTestPasswordManager::initPasswordManagerBase()
     ASSERT_TRUE(rl.waitForFinishOrTimeout(MAX_TIMEOUT));
     ASSERT_NE(mPWMBaseNodeHandle, UNDEF);
 }
+
+void SdkTestPasswordManager::removePWMNodes()
+{
+    const std::unique_ptr<MegaNode> pwmBaseNode(megaApi[0]->getNodeByHandle(mPWMBaseNodeHandle));
+    ASSERT_TRUE(pwmBaseNode);
+    ASSERT_NO_FATAL_FAILURE(purgeTree(0, pwmBaseNode.get(), false));
+}
