@@ -29,6 +29,9 @@ protected:
     // Use mApi[0] and mApi[1]
     void removeShareAtoB(MegaNode* node);
 
+    // Reset credential between two accounts if contact found
+    void resetCredentialsIfContactFound(const unsigned i, const unsigned j);
+
     // Reset credential between two accounts
     void resetCredential(unsigned apiIndexA, unsigned apiIndexB);
 
@@ -86,7 +89,7 @@ void SdkTestShare::createShareAtoB(MegaNode* node,
     // convinience
     auto& apiA = mApi[partyA.apiIndex];
     auto& apiB = mApi[partyB.apiIndex];
-
+    apiA.nodeUpdated = apiB.nodeUpdated = false;
     apiA.mOnNodesUpdateCompletion = createOnNodesUpdateLambda(node->getHandle(),
                                                               MegaNode::CHANGE_TYPE_OUTSHARE,
                                                               apiA.nodeUpdated);
@@ -133,6 +136,17 @@ void SdkTestShare::removeShareAtoB(MegaNode* node)
     mApi[0].nodeUpdated = mApi[1].nodeUpdated = false;
 }
 
+void SdkTestShare::resetCredentialsIfContactFound(const unsigned i, const unsigned j)
+{
+    std::unique_ptr<MegaUser> c1(mApi[i].megaApi->getContact(mApi[j].email.c_str()));
+    std::unique_ptr<MegaUser> c2(mApi[j].megaApi->getContact(mApi[i].email.c_str()));
+    if (!c1 || !c2)
+    {
+        return;
+    }
+    ASSERT_NO_FATAL_FAILURE(resetCredential(i, j));
+}
+
 void SdkTestShare::resetCredential(unsigned apiIndexA, unsigned apiIndexB)
 {
     if (areCredentialsVerified(apiIndexA, mApi[apiIndexB].email))
@@ -406,41 +420,46 @@ TEST_F(SdkTestShare, TestSharesContactVerification)
         60 * 1000))
         << "Cannot decrypt inshare in B account.";
 
-    /*
-     * TODO: uncomment this block to test "Reset credentials" when SDK supports APIv3 for up2/upv
-     * commands
-     *
-     * This test is prone to a race condition that may result on having no inshare, but unverified
-     * inshare.
-     *
-     * It happens when the client receives a "pk" action packet after reset credentials. Why that
-     * "pk"? because currently the SDK cannot differentiate between action packets related to its
-     * own user's attribute updates
-     * (^!keys) and other client's updates. In consequence, if the action packet is received before
-     * the response to the "upv", the SDK will fetch the attribute ("uga") and upon receiving the
-     * value, it will reapply the promotion of the outshare, sending a duplicated "pk" for the same
-     * share handle.
-     *
-     * This race between the sc and cs channels will be removed when the SDK adds support for the
-     * APIv3 / sn-tagging, since the "upv" will be matched with the corresponding action packet,
-     * eliminating the race.
-     * */
+    // Auxiliar function to reset both accounts credentials verification
+    auto resetAllCredentials = [this]()
+    {
+        ASSERT_NO_FATAL_FAILURE(resetCredentials(0, mApi[1].email));
+        ASSERT_NO_FATAL_FAILURE(resetCredentials(1, mApi[0].email));
+        ASSERT_FALSE(areCredentialsVerified(0, mApi[1].email));
+        ASSERT_FALSE(areCredentialsVerified(1, mApi[0].email));
+    };
 
-    // // Reset credentials
-    // LOG_verbose << "TestSharesContactVerification :  Reset credentials";
-    // ASSERT_NO_FATAL_FAILURE(resetAllCredentials());
-    // // Established share remains in the same status.
-    // ASSERT_TRUE(WaitFor([this]() { return
-    // unique_ptr<MegaShareList>(megaApi[0]->getOutShares())->size() == 1; }, 60*1000));
-    // ASSERT_TRUE(WaitFor([this]() { return
-    // unique_ptr<MegaShareList>(megaApi[0]->getUnverifiedOutShares())->size() == 0; }, 60*1000));
-    // ASSERT_TRUE(WaitFor([this]() { return
-    // unique_ptr<MegaShareList>(megaApi[1]->getInSharesList())->size() == 1; }, 60*1000));
-    // ASSERT_TRUE(WaitFor([this]() { return
-    // unique_ptr<MegaShareList>(megaApi[1]->getUnverifiedInShares())->size() == 0; }, 60*1000));
-    // inshareNode.reset(megaApi[1]->getNodeByHandle(nh));
-    // ASSERT_NE(inshareNode.get(), nullptr);
-    // ASSERT_TRUE(inshareNode->isNodeKeyDecrypted()) << "Cannot decrypt inshare in B account.";
+    // Reset credentials
+    LOG_verbose << "TestSharesContactVerification :  Reset credentials";
+    ASSERT_NO_FATAL_FAILURE(resetAllCredentials());
+    // Established share remains in the same status.
+    ASSERT_TRUE(WaitFor(
+        [this]()
+        {
+            return unique_ptr<MegaShareList>(megaApi[0]->getOutShares())->size() == 1;
+        },
+        60 * 1000));
+    ASSERT_TRUE(WaitFor(
+        [this]()
+        {
+            return unique_ptr<MegaShareList>(megaApi[0]->getUnverifiedOutShares())->size() == 0;
+        },
+        60 * 1000));
+    ASSERT_TRUE(WaitFor(
+        [this]()
+        {
+            return unique_ptr<MegaShareList>(megaApi[1]->getInSharesList())->size() == 1;
+        },
+        60 * 1000));
+    ASSERT_TRUE(WaitFor(
+        [this]()
+        {
+            return unique_ptr<MegaShareList>(megaApi[1]->getUnverifiedInShares())->size() == 0;
+        },
+        60 * 1000));
+    inshareNode.reset(megaApi[1]->getNodeByHandle(nh));
+    ASSERT_NE(inshareNode.get(), nullptr);
+    ASSERT_TRUE(inshareNode->isNodeKeyDecrypted()) << "Cannot decrypt inshare in B account.";
 
     // Remove share
     LOG_verbose << "TestSharesContactVerification :  Remove shared folder from A to B";
@@ -1079,8 +1098,8 @@ TEST_F(SdkTestShareOrder, GetOutSharesOrUnverifiedOutSharesOrderedByCreationTime
     megaApi[2]->setManualVerificationFlag(true);
 
     // Ensure no account has the other verified from previous unfinished tests.
-    ASSERT_NO_FATAL_FAILURE(resetCredential(0, 1));
-    ASSERT_NO_FATAL_FAILURE(resetCredential(0, 2));
+    ASSERT_NO_FATAL_FAILURE(resetCredentialsIfContactFound(0, 1));
+    ASSERT_NO_FATAL_FAILURE(resetCredentialsIfContactFound(0, 2));
 
     LOG_info << "Invite from account 0 to 1 and verify credential";
     ASSERT_NO_FATAL_FAILURE(addContactsAndVerifyCredential(0, 1));
@@ -1149,55 +1168,79 @@ TEST_F(SdkTestShareOrder, GetOutSharesOrUnverifiedOutSharesOrderedByCreationTime
 
 TEST_F(SdkTestShare, TestSharesPermission)
 {
-    LOG_info << "___TEST TestSharesPermission___";
-
+    static const auto logPre = getLogPrefix();
     ASSERT_NO_FATAL_FAILURE(getAccountsForTest(3));
 
     // Ensure no account has the other verified from previous unfinished tests.
-    ASSERT_NO_FATAL_FAILURE(resetCredential(0, 1));
-    ASSERT_NO_FATAL_FAILURE(resetCredential(0, 2));
+    ASSERT_NO_FATAL_FAILURE(resetCredentialsIfContactFound(0, 1));
+    ASSERT_NO_FATAL_FAILURE(resetCredentialsIfContactFound(0, 2));
 
-    LOG_info << "Invite from account 0 to 1 and verify credential";
+    LOG_info << logPre
+             << "#### Test preconditions. Invite from account 0 to 1 and verify "
+                "credential ####";
     ASSERT_NO_FATAL_FAILURE(addContactsAndVerifyCredential(0, 1));
-    LOG_info << "Invite from account 0 to 2 and verify credential";
+    LOG_info << logPre
+             << "#### Test preconditions. Invite from account 0 to 2 and verify "
+                "credential ####";
     ASSERT_NO_FATAL_FAILURE(addContactsAndVerifyCredential(0, 2));
 
     // Root node
     std::unique_ptr<MegaNode> remoteRootNode(megaApi[0]->getRootNode());
     ASSERT_NE(remoteRootNode.get(), nullptr);
 
-    LOG_info << "Create folders";
+    LOG_info << logPre << "#### Test preconditions. Create folders ####";
     auto [handle1, shareNode1] = createFolder(0, "share1", remoteRootNode.get());
     auto [handle2, shareNode2] = createFolder(0, "share2", remoteRootNode.get());
     ASSERT_THAT(shareNode1, testing::NotNull());
     ASSERT_THAT(shareNode2, testing::NotNull());
 
-    LOG_info << "Share (full access) folder from account 0 to account 1 share node 1";
+    auto waitForNode = [this](const unsigned idx,
+                              const std::string& sharedPath,
+                              std::shared_ptr<MegaNode>& sharedNode) -> std::function<bool()>
+    {
+        // Wait until node attr's is decrypted, otherwise getNodeByPath could not retrieve
+        // it, even if it has already been received
+        return [=, &sharedNode]()
+        {
+            sharedNode.reset(megaApi[idx]->getNodeByPath(sharedPath.c_str()));
+            return !!sharedNode;
+        };
+    };
+
+    LOG_info << logPre
+             << "#### Test1 Share (full access) folder from account 0 to account "
+                "1 share node 1 ####";
     ASSERT_NO_FATAL_FAILURE(createShareAtoB(shareNode1.get(), true, true, MegaShare::ACCESS_FULL));
     {
+        std::shared_ptr<MegaNode> sharedNode;
         string sharedPath = megaApi[0]->getMyEmail();
         sharedPath.append(":share1");
         RequestTracker listener{megaApi[1].get()};
-        auto sharedNode = megaApi[1]->getNodeByPath(sharedPath.c_str());
-        megaApi[1]->setNodeLabel(sharedNode, 1, &listener);
+        ASSERT_TRUE(WaitFor(waitForNode(1, sharedPath, sharedNode), 60 * 1000))
+            << "Cannot get outshare in A account.";
+        megaApi[1]->setNodeLabel(sharedNode.get(), 1, &listener);
         ASSERT_TRUE(API_OK == listener.waitForResult());
         ASSERT_TRUE(WaitFor(
             [this]() -> bool
             {
-                auto sharedNode = megaApi[0]->getNodeByPath("/share1");
+                std::unique_ptr<MegaNode> sharedNode(megaApi[0]->getNodeByPath("/share1"));
                 return (1 == sharedNode->getLabel());
             },
             20 * 1000));
     }
 
-    LOG_info << "Share (Read and Write) folder from account 0 to account 2 share node 2";
+    LOG_info << logPre
+             << "#### Test2. Share (Read and Write) folder from account 0 to "
+                "account 2 share node 2 ####";
     ASSERT_NO_FATAL_FAILURE(createShareAtoB(shareNode2.get(), {0, true}, {2, true}));
     {
+        std::shared_ptr<MegaNode> sharedNode;
         string sharedPath = megaApi[0]->getMyEmail();
         RequestTracker listener{megaApi[2].get()};
         sharedPath.append(":share2");
-        auto sharedNode = megaApi[2]->getNodeByPath(sharedPath.c_str());
-        megaApi[2]->setNodeLabel(sharedNode, 1, &listener);
+        ASSERT_TRUE(WaitFor(waitForNode(2, sharedPath, sharedNode), 60 * 1000))
+            << "Cannot get outshare in A account.";
+        megaApi[2]->setNodeLabel(sharedNode.get(), 1, &listener);
         ASSERT_TRUE(API_EACCESS == listener.waitForResult());
     }
 }
