@@ -83,6 +83,7 @@ extern std::string USER_AGENT;
 extern bool gResumeSessions;
 extern bool gScanOnly;
 extern bool gManualVerification;
+extern bool gFreeAccounts;
 
 LogStream out();
 
@@ -771,6 +772,14 @@ struct StandardClient : public MegaApp
     void request_response_progress(m_off_t a, m_off_t b) override;
     void threadloop();
 
+    void updateClientDowaitDs(const dstime lastClientDoWait);
+    dstime consumeClientDowaitDs(const dstime timeGranularity = 1);
+    void resetClientDowaitDs();
+
+private:
+    dstime mClientDowaitDs{};
+
+public:
     static bool debugging;  // turn this on to prevent the main thread timing out when stepping in the MegaClient
 
     template <class PROMISE_VALUE>
@@ -908,6 +917,7 @@ struct StandardClient : public MegaApp
             sendPutnodesOfUpload(
                 t->client,
                 t->uploadhandle,
+                "",
                 *t->ultoken,
                 t->filekey,
                 source,
@@ -1179,6 +1189,7 @@ struct StandardClient : public MegaApp
     bool waitFor(std::function<bool(StandardClient&)> predicate,
                  std::chrono::seconds timeout,
                  std::chrono::milliseconds sleepIncrement = std::chrono::milliseconds(500));
+    bool waitForSyncTotalStallsStateUpdateTrue(const std::chrono::seconds timeout);
     bool match(const Node& destination, const Model::ModelNode& source) const;
     bool makeremotenodes(const string& prefix, int depth, int fanout);
     bool backupOpenDrive(const fs::path& drivePath);
@@ -1222,6 +1233,13 @@ struct StandardClient : public MegaApp
                         LocalPath& sourcePath,
                         LocalPath& targetPath);
 
+    /**
+     * @brief Deletes a file using the client fileAccess.
+     *
+     * It uses DEFAULTWAIT for retries if the operation ends with a transient error.
+     */
+    void unlinklocal(const LocalPath& localPath);
+
     /**
      * @brief Checks for synchronization stall issues related to a specific BackupId.
      *
