@@ -74,6 +74,22 @@ void setTestDataDir(const fs::path& dataDir);
  */
 void copyFileFromTestData(fs::path filename, fs::path destination = ".");
 
+/**
+ * @brief Returns a SHA-256 hash of the file.
+ */
+std::string hashFile(const fs::path& filePath);
+
+/**
+ * @brief Returns a SHA-256 hash of the file with a human-readable format.
+ */
+std::string hashFileHex(const fs::path& filePath);
+
+/**
+ * @brief Creates a file of a given size with random, printable data. It throws if the file cannot
+ * be opened.
+ */
+void createRandomFile(const fs::path& filePath, const size_t fileSizeBytes);
+
 /**
  * @brief Creates a file of a given size. It throws if the file cannot be opened
  */
@@ -82,7 +98,9 @@ void createFile(const fs::path& filePath, const size_t fileSizeBytes);
 /**
  * @brief Creates a file with the given contents. It throws if the file cannot be opened
  */
-void createFile(const fs::path& filePath, const std::string_view contents);
+void createFile(const fs::path& filePath,
+                const std::string_view contents,
+                std::optional<fs::file_time_type> customMtime = std::nullopt);
 
 /**
  * @brief Appends data to a file with the given contents. It throws if the file cannot be opened
@@ -102,7 +120,9 @@ class LocalTempFile
 {
 public:
     LocalTempFile(const fs::path& _filePath, const size_t fileSizeBytes);
-    LocalTempFile(const fs::path& _filePath, const std::string_view contents);
+    LocalTempFile(const fs::path& _filePath,
+                  const std::string_view contents,
+                  std::optional<fs::file_time_type> customMtime = std::nullopt);
     ~LocalTempFile();
 
     // Delete copy constructors -> Don't allow many objects to remove the same file
