@@ -727,7 +727,7 @@ TEST_F(SqliteDBTest, RootPath)
 TEST(LocalPath, AppendWithSeparator)
 {
     LocalPath source;
-    LocalPath target;
+    LocalPath target = LocalPath::fromRelativePath("");
 
     // Doesn't add a separator if the target is empty.
     source = LocalPath::fromRelativePath("a");
@@ -1428,49 +1428,6 @@ TEST(ScopedHelpers, ScopedDestructor)
     EXPECT_EQ(x, 4);
 }
 
-TEST(ScopedHelpers, ScopedLengthRestorer)
-{
-    // Test with local path.
-    {
-        auto x = LocalPath::fromAbsolutePath("x");
-        auto originalSize = SizeTraits<LocalPath>::size(x);
-
-        {
-            auto r = makeScopedSizeRestorer(x);
-            x.appendWithSeparator(LocalPath::fromRelativePath("y"), true);
-            EXPECT_NE(SizeTraits<LocalPath>::size(x), originalSize);
-        }
-
-        EXPECT_EQ(SizeTraits<LocalPath>::size(x), originalSize);
-    }
-
-    // Test with vector.
-    {
-        std::vector<std::string> x(1, "foo");
-
-        auto originalSize = x.size();
-
-        {
-            auto r = makeScopedSizeRestorer(x);
-            x.emplace_back("bar");
-        }
-
-        EXPECT_EQ(x.size(), originalSize);
-    }
-
-    // Test with explicit new size given.
-    {
-        std::string x;
-
-        {
-            auto r = makeScopedSizeRestorer(x, 32);
-            EXPECT_EQ(x.size(), 32);
-        }
-
-        EXPECT_TRUE(x.empty());
-    }
-}
-
 TEST(ScopedHelpers, ScopedValue)
 {
     const std::string originalValue = "before";
@@ -1817,3 +1774,193 @@ TEST(RangeTest, VerifySingleElementRange)
     EXPECT_EQ(count, 1);
     EXPECT_EQ(valueCollected, 7u);
 }
+
+struct FileAccessTest: ::testing::Test
+{
+    FileAccessTest():
+        Test(),
+        mFSAccess(),
+        mName(LocalPath::fromAbsolutePath("file"))
+    {}
+
+    // Called before any test in the fixture is executed.
+    void SetUp() override
+    {
+        // Remove the file if it's present after a previous test run.
+        ASSERT_TRUE(mFSAccess.unlinklocal(mName) || !mFSAccess.target_exists);
+    }
+
+    // Convenience.
+    ::mega::FSLogging NO_LOGGING = ::mega::FSLogging::noLogging;
+
+    // So we can get our hands on a FileAccess instance.
+    FSACCESS_CLASS mFSAccess;
+
+    // The name of our test file.
+    ::mega::LocalPath mName;
+}; // FileAccessTest
+
+TEST_F(FileAccessTest, OpenForReadWriteSucceeds)
+{
+    // So we can open a file.
+    auto fileAccess = mFSAccess.newfileaccess(false);
+
+    // Sanity.
+    ASSERT_TRUE(fileAccess);
+
+    // Opening for reading and writing should create a new file if necessary.
+    EXPECT_TRUE(fileAccess->fopen(mName, true, true, NO_LOGGING));
+}
+
+TEST_F(FileAccessTest, OpenEquivalence)
+{
+    auto fileAccess0 = mFSAccess.newfileaccess(false);
+    auto fileAccess1 = mFSAccess.newfileaccess(false);
+
+    // Sanity.
+    ASSERT_TRUE(fileAccess0);
+    ASSERT_TRUE(fileAccess1);
+
+    // Create a new file.
+    ASSERT_TRUE(fileAccess0->fopen(mName, true, true, NO_LOGGING));
+
+    // Open an existing file.
+    EXPECT_TRUE(fileAccess1->fopen(mName, true, true, NO_LOGGING));
+
+    // Convenience.
+    auto& lhs = *fileAccess0;
+    auto& rhs = *fileAccess1;
+
+    // Make sure selected state is equivalent.
+    EXPECT_EQ(lhs.fopenSucceeded, rhs.fopenSucceeded);
+    EXPECT_EQ(lhs.size, rhs.size);
+    EXPECT_EQ(lhs.mtime, rhs.mtime);
+    EXPECT_EQ(lhs.fsid, rhs.fsid);
+    EXPECT_EQ(lhs.type, rhs.type);
+    EXPECT_EQ(lhs.mIsSymLink, rhs.mIsSymLink);
+}
+
+TEST(IP, is_valid_ipv4_address_fails)
+{
+    ASSERT_FALSE(isValidIPv4Address(""));
+    ASSERT_FALSE(isValidIPv4Address("1"));
+    ASSERT_FALSE(isValidIPv4Address("1.2"));
+    ASSERT_FALSE(isValidIPv4Address("1.2.3"));
+    ASSERT_FALSE(isValidIPv4Address("::1"));
+}
+
+TEST(IP, is_valid_ipv4_address_succeeds)
+{
+    ASSERT_TRUE(isValidIPv4Address("192.168.0.1"));
+}
+
+TEST(IP, is_valid_ipv6_address_fails)
+{
+    ASSERT_FALSE(isValidIPv6Address(""));
+    ASSERT_FALSE(isValidIPv6Address("192.168.0.1"));
+    ASSERT_FALSE(isValidIPv6Address("0"));
+    ASSERT_FALSE(isValidIPv6Address("::q"));
+}
+
+TEST(IP, is_valid_ipv6_address_succeeds)
+{
+    ASSERT_TRUE(isValidIPv6Address("2001:db8:3333:4444:5555:6666:7777:8888"));
+    ASSERT_TRUE(isValidIPv6Address("2001:db8::"));
+    ASSERT_TRUE(isValidIPv6Address("::1234:5678"));
+    ASSERT_TRUE(isValidIPv6Address("2001:db8:3333:4444:5555:6666:1.2.3.4"));
+    ASSERT_TRUE(isValidIPv6Address("2001:db8::1234:5678:5.6.7.8"));
+    ASSERT_TRUE(isValidIPv6Address("::11.22.33.44"));
+}
+
+// Using string_vector to avoid parsing problems with GTest.
+TEST(DNS, cache_resolved_urls_fails)
+{
+    CurlHttpIO io;
+
+    // Not enough IPs for each URI.
+    EXPECT_LT(io.cacheresolvedurls(string_vector(1), string_vector(1)), 0);
+
+    // Not enough URIs for each IP.
+    EXPECT_LT(io.cacheresolvedurls(string_vector(1), string_vector(4)), 0);
+
+    // Multiple URIs and no IPs.
+    EXPECT_LT(io.cacheresolvedurls(string_vector(2), string_vector()), 0);
+
+    // Make sure the cache remains empty.
+    EXPECT_TRUE(io.getCachedDNSEntries().empty());
+}
+
+TEST(DNS, cache_resolved_urls_succeeds)
+{
+    CurlHttpIO io;
+
+    // Expected DNS cache entries.
+    std::map<std::string, DNSEntry> expected = {{"a.com", DNSEntry{"1.2.3.4", "::1"}},
+                                                {"b.com", DNSEntry{"2.3.4.5", "::2"}}}; // expected
+
+    std::vector<std::string> uris = {"https://a.com", "https://b.com"};
+    std::vector<std::string> ips = {"1.2.3.4", "::1", "2.3.4.5", "::2"};
+
+    // Each URI is associated with a valid IPv4 and IPv6 address.
+    EXPECT_EQ(io.cacheresolvedurls(uris, ips), 0);
+
+    // Make sure the DNS cache was updated as expected.
+    EXPECT_EQ(expected, io.getCachedDNSEntries());
+
+    // Each URI is associated with an invalid IP.
+    //
+    // a.com has an invalid IPv4 address.
+    // b.com has an invalid IPv6 address.
+    ips[0] = "badV4";
+    ips[3] = "badV6";
+
+    EXPECT_EQ(io.cacheresolvedurls(uris, ips), 2);
+
+    // Make sure previously valid IPs were cleared.
+    expected["a.com"].ipv4.clear();
+    expected["b.com"].ipv6.clear();
+
+    EXPECT_EQ(expected, io.getCachedDNSEntries());
+
+    // Each URI isn't associated with any valid IP address.
+    ips[1] = ips[3];
+    ips[2] = ips[0];
+
+    EXPECT_EQ(io.cacheresolvedurls(uris, ips), 4);
+
+    // Make sure URIs remain and that any valid IPs were cleared.
+    expected["a.com"].ipv6.clear();
+    expected["b.com"].ipv4.clear();
+
+    EXPECT_EQ(expected, io.getCachedDNSEntries());
+
+    // A new URI isn't associated with any valid IP addresses.
+    uris = {"c.com"};
+    ips = {"badV4", "badV6"};
+
+    EXPECT_EQ(io.cacheresolvedurls(uris, ips), 2);
+
+    // Make sure no entry was added to the DNS cache.
+    EXPECT_EQ(expected, io.getCachedDNSEntries());
+
+    // Two new URIs only have a single valid IP each.
+    uris = {"https://d.com", "https://e.com"};
+    ips = {"4.5.6.7", "badV6", "badV4", "::3"};
+
+    EXPECT_EQ(io.cacheresolvedurls(uris, ips), 2);
+
+    // Make sure each URI was added to the cache.
+    expected["d.com"].ipv4 = ips[0];
+    expected["e.com"].ipv6 = ips[3];
+
+    EXPECT_EQ(expected, io.getCachedDNSEntries());
+
+    // Invalid URIs are skipped.
+    uris = {"z"};
+    ips = {"4.3.2.1", "::4"};
+
+    EXPECT_EQ(io.cacheresolvedurls(uris, ips), 0);
+
+    // Make sure the bad URI wasn't added to the cache.
+    EXPECT_EQ(expected, io.getCachedDNSEntries());
+}
