@@ -1,5 +1,9 @@
 #include "executable_dir.h"
 
+// headers from gfxworker
+#include "logger.h"
+
+// headers from sdk
 #include "megaapi.h"
 
 #include <gtest/gtest.h>
@@ -12,6 +16,7 @@ int main (int argc, char *argv[])
     // log
     ::mega::MegaApi::setLogLevel(mega::MegaApi::LOG_LEVEL_MAX);
 
+    ::mega::gfx::MegaFileLogger::get().initialize(".", "gfxworker_test_integration.log", false);
     // test
     testing::InitGoogleTest(&argc, argv);
     return RUN_ALL_TESTS();
